//=========================================================================
//Copyright LizardOfOz.
//=========================================================================

::CreateAoE <- function(origin, radius, applyDamageFunc, applyPushFunc)
{
    foreach(target in GetAlivePlayers())
    {
        local deltaVector = target.GetOrigin() - origin;
        local distance = deltaVector.Norm();
        if (distance > radius)
            continue;

        applyPushFunc(target, deltaVector, distance);
        applyDamageFunc(target, deltaVector, distance);
    }

    local target = null;
    while (target = Entities.FindByClassname(target, "obj_*"))
    {
        local deltaVector = target.GetOrigin() - origin;
        local distance = deltaVector.Norm();
        if (distance > radius)
            continue;

        applyDamageFunc(target, deltaVector, distance);
    }
}

::CreateAoEAABB <- function(origin, min, max, applyDamageFunc, applyPushFunc)
{
    local min = origin + min;
    local max = origin + max;
    foreach(target in GetAlivePlayers())
    {
        local targetOrigin = target.GetOrigin();
        if (targetOrigin.x >= min.x
            && targetOrigin.y >= min.y
            && targetOrigin.z >= min.z
            && targetOrigin.x <= max.x
            && targetOrigin.y <= max.y
            && targetOrigin.z <= max.z)
            {
                local deltaVector = targetOrigin - origin;
                local distance = deltaVector.Norm();
                applyPushFunc(target, deltaVector, distance);
                applyDamageFunc(target, deltaVector, distance);
            }
    }

    local target = null;
    while (target = Entities.FindByClassname(target, "obj_*"))
    {
        local targetOrigin = target.GetOrigin();
        if (targetOrigin.x >= min.x
            && targetOrigin.y >= min.y
            && targetOrigin.z >= min.z
            && targetOrigin.x <= max.x
            && targetOrigin.y <= max.y
            && targetOrigin.z <= max.z)
            {
                local deltaVector = targetOrigin - center;
                local distance = deltaVector.Norm();
                applyDamageFunc(target, deltaVector, distance);
            }
    }
}

::clampCeiling <- function(valueA, valueB)
{
    if (valueA < valueB)
        return valueA;
    return valueB;
}

::clampFloor <- function(valueA, valueB)
{
    if (valueA > valueB)
        return valueA;
    return valueB;
}

::clamp <- function(value, min, max)
{
    if (value < min)
        return min;
    if (value > max)
        return max;
    return value;
}

::safeget <- function(table, field, defValue)
{
    return field in table ? table[field] : defValue;
}

::SetPersistentVar <- function(name, value)
{
    local persistentVars = tf_gamerules.GetScriptScope();
    persistentVars[name] <- value;
}

::GetPersistentVar <- function(name, defValue = null)
{
    local persistentVars = tf_gamerules.GetScriptScope();
    return name in persistentVars ? persistentVars[name] : defValue;
}

::ShuffleArray <- function(array)
{
    local currentIndex = array.len() - 1;
    local swap;
    local randomIndex;

    while (currentIndex > 0)
    {
        randomIndex = RandomInt(0, currentIndex);
        currentIndex -= 1;

        swap = array[currentIndex];
        array[currentIndex] = array[randomIndex];
        array[randomIndex] = swap;
    }

    return array;
}

::RunWithDelay <- function (scope, delay, func, ...)
{
    //Thanks Mr.Burguers and Ficool2 for help
    local tmpEnt = Entities.CreateByClassname("point_template");
    local name = tmpEnt.GetScriptId();
    local code = format("main_script.%s()", name);
    main_script[name] <- function()
    {
        local err = null;
        try { func.acall([scope].extend(vargv)); }
        catch (e) { err = e; }
        delete main_script[name];
        if (err) throw err;
    }
    SetPropBool(tmpEnt, "m_bForcePurgeFixedupStrings", true);
    SetPropString(tmpEnt, "m_iName", code);
    EntFireByHandle(main_script_entity, "RunScriptCode", code, delay, null, null);
    EntFireByHandle(tmpEnt, "Kill", null, delay, null, null)
}

::RunWithDelayPlayer <-  function(scope, delay, func, player, ...)
{
    //Thanks Mr.Burguers and Ficool2 for help
    local tmpEnt = Entities.CreateByClassname("point_template");
    local name = tmpEnt.GetScriptId();
    local code = format("main_script.%s()", name);
    main_script[name] <- function()
    {
        local err = null;
        try
        {
            if (IsValidPlayer(player))
                func.acall([scope, player].extend(vargv));
        }
        catch (e) { err = e; }
        delete main_script[name];
        if (err) throw err;
    }
    SetPropBool(tmpEnt, "m_bForcePurgeFixedupStrings", true);
    SetPropString(tmpEnt, "m_iName", code);
    EntFireByHandle(main_script_entity, "RunScriptCode", code, delay, null, null);
    EntFireByHandle(tmpEnt, "Kill", null, delay, null, null)
}