::mercModelPath <- "models/player/mercenary.mdl";

PrecacheModel(mercModelPath);
PrecacheScriptSound("CaptureFlag.FlagSpawn")

PrecacheSound("vo/mercenary_paincrticialdeath01.mp3")
PrecacheSound("vo/mercenary_paincrticialdeath02.mp3")
PrecacheSound("vo/mercenary_paincrticialdeath03.mp3")
PrecacheSound("vo/mercenary_paincrticialdeath04.mp3")

vo_death_sounds <- ["vo/mercenary_paincrticialdeath01.mp3", "vo/mercenary_paincrticialdeath02.mp3", "vo/mercenary_paincrticialdeath03.mp3", "vo/mercenary_paincrticialdeath04.mp3"]

OnGameEvent("player_spawn", 0, function(player, params)
{
    player.SetScriptOverlayMaterial("");

    if (redTeamOnly && player.GetTeam() != TF_TEAM_RED)
    {
        player.SwitchTeam(TF_TEAM_RED);
        player.ForceRespawn();
    }
});

OnGameEventPost("player_spawn", 0, function (player, params)
{
    //Delete weapons and cosmetics
    local itemsToKill = [];
    for (local item = player.FirstMoveChild(); item != null; item = item.NextMovePeer())
        if (startswith(item.GetClassname(), "tf_we"))
            itemsToKill.push(item);
    foreach (item in itemsToKill)
        item.Kill();

    SetPropInt(player, "m_Shared.m_bShieldEquipped", 0);
    //Equip new weapons
    EquipWeapon(player, GetWeaponTypeByName("crowbar"));
    player.Weapon_Switch(EquipWeapon(player, GetCurrentGunGameWeapon(player)));

    //Refresh model, movespeed and HP
    RefreshCharacter(player, GetCurrentGunGameWeapon(player), false);

    //Add Merc's attributes
    player.AddCustomAttribute("cancel falling damage", 1, -1);
    player.AddCustomAttribute("crit mod disabled hidden", 0, -1);
    player.AddCustomAttribute("voice pitch scale", 0, -1);
    player.AddCustomAttribute("no double jump", 1, -1);

    //Steam-bound clothing color
    SetEntityColor(player, CalcMercColor(player));

    //On-spawn invul
    SetPropInt(player, "m_bForcedSkin", 1);
    SetPropInt(player, "m_nForcedSkin", 5);
    player.AddCondEx(TF_COND_INVULNERABLE, GetRoundState() == 3 ? 5 : 3, null);
    RunWithDelayPlayer(this, GetRoundState() == 3 ? 5 : 3, function(player)
    {
        SetPropInt(player, "m_nForcedSkin", 4);
    }, player)

    //Spawn sound
    if (GetRoundState() >= 4)
        CreateAoE(player.GetOrigin(), 500, function(target, deltaVector, distance) {
            for (local i = 0; i < 10; i++)
            EmitSoundEx({
                sound_name = "CaptureFlag.FlagSpawn",
                volume = 1,
                sound_level = 150,
                flags = 1,
                channel = i,
                entity = player,
                speaker_entity = player
            });
        }, function(target, deltaVector, distance) {})
});

function RefreshCharacter(player, weaponTT, keepCurrentHP = true)
{
    local playerClass = "playerclass" in weaponTT ? weaponTT.playerclass : player.GetPlayerClass();
    local oldHP = player.GetHealth();
    player.SetPlayerClass(playerClass);
    player.AddCustomAttribute("move speed bonus",
        1.04 / MERC_MOVESPEED_MULTIPLIER[playerClass],
        -1);

    player.SetCustomModelWithClassAnimations(mercModelPath);

    player.SetHealth(keepCurrentHP ? oldHP : 150);
    player.SetMaxHealth(150);
    player.RemoveCustomAttribute("max health additive penalty");
    player.AddCustomAttribute("max health additive penalty", MERC_HP_OFFSETS[playerClass], -1);

    RefreshModels(player);
}

OnGameEvent("player_death", 0, function(player, params)
{
    EmitSoundEx({
        sound_name = vo_death_sounds[RandomInt(0, vo_death_sounds.len() - 1)],
        entity = player,
        sound_level = 90,
    });
    RunWithDelayPlayer(this, 3, function(player)
    {
        player.ForceRespawn();
    }, player);
});

OnGameEvent("player_death_post", 0, function(player, params)
{
    local ragdoll = GetPropEntity(player, "m_hRagdoll");
    SetPropBool(ragdoll, "m_bGib", false);
    SetPropInt(ragdoll, "m_clrRender", 4444444);
});

AddTimer(-1, 0, function ()
{
    foreach (player in GetAlivePlayers())
    {
        if (GetPropInt(player, "m_nButtons") & IN_ATTACK)
        {
            player.RemoveCond(TF_COND_INVULNERABLE);
            SetPropInt(player, "m_nForcedSkin", 4);
        }
    }
})

::SetEntityColor <- function(entity, rgb)
{
    local color = rgb[0] | (rgb[1] << 8) | (rgb[2] << 16);
    SetPropInt(entity, "m_clrRender", color);
}

::CalcMercColor <- function(player)
{
    local steamId = GetPropString(player, "m_szNetworkIDString");
    local indexStr = steamId.len() > 5 ? steamId.slice(5) : "4444444";
    local index = indexStr.tointeger();
    return [index % 256, (index >> 8) % 256, (index >> 16) % 256, 255];
}